/*************************************************************/
/*                                                           */
/*        Programme gnu_13_c.c                               */
/*                                                           */
/* Ce programme indique comment passer en mode 13h en C gnu. */
/* Il a t crit  pour que l'on puisse s'en inspirer,  pour */
/* accder  la carte grahique,  ou  la mmoire basse, quel */
/* que soit le problme pos.  Pour indiquer une utilisation */
/* possibles des routines de base, il offre un petit feu.    */
/*                                                           */
/* Compilateur: C gnu djgpp pour DOS                         */
/*                                                           */
/* (c) Copyright Opie Pcheux Mars 2000                      */
/*    ABC de la Programmation                                */
/*  http://www.multimania.com/opecheux                       */
/*  mailto:opecheux@hop.multimania.com                       */
/*                                                           */
/*************************************************************/


#include <dpmi.h> /* __dpmi_int __dpmi_regs _farsetsel */
#include <pc.h> /* kbhit outportb inportb */
#include <go32.h> /* dos_ds */
#include <sys/farptr.h> /* _farnspokeb _parnspeekb */
#include <stdlib.h> /* random */


/*************************************************************/
/* Entte des fonctions permettant l'accs au mode 13h       */
/*************************************************************/

void mode13(void);
void mode3(void);
void putpixel(int x, int y, unsigned char couleur);
unsigned char getpixel(int x, int y);
void setcolor(unsigned char couleur, unsigned char rouge, unsigned char vert,
              unsigned char bleu);
void setcolorsuite(unsigned char rouge, unsigned char vert,
              unsigned char bleu);
void vsync(void);



/*************************************************************/
/* Programme d'exemple: petit feu                            */
/*************************************************************/
int main(void)
{
  int x, /* Abscisses des points */
      y, /* Ordonns des points */
      hauteur, /* hauteur du feu */
      couleur; /* Numro dans la palette */

  /* Passage en mode graphique */
  mode13();

  /* Programmation de la palette */
  for (couleur=0; couleur<64; couleur++) setcolor(couleur,couleur,0,0);
  for (couleur=0; couleur<128; couleur++) setcolorsuite(63,couleur>>1,0);
  for (couleur=0; couleur<64; couleur++) setcolorsuite(0,0,63);

  /* dessins du feu */
  hauteur=200;
  do
  {
    /* Ligne du bas: couleurs au hazard */
    for (x=80; x<240; x++) putpixel(x,200,random() & 0xFF);

    /* Autres lignes: voir algorithme */
    for (y=hauteur; y<200; y++) for (x=80; x<240; x++)
    {
      couleur=(getpixel(x,y+1)+getpixel(x-1,y+1)+
        getpixel(x+1,y+1)+getpixel(x,y+2))/4;
      if (couleur) couleur--;
      putpixel(x,y,couleur);
    }
    vsync();
    /* voir si le dessin est mont */
    for (x=80; x<240; x++)
      if ((getpixel(x,hauteur)+getpixel(x-1,hauteur)+
        getpixel(x+1,hauteur)+getpixel(x,hauteur+1))/4) hauteur--;

  }
  while (!kbhit()); /* attendre l'appui sur une touche */

  /* Retour en mode texte */
  mode3();
}



/*************************************************************/
/*************************************************************/
/**                                                         **/
/**           Fonctions graphiques de base                  **/
/**                                                         **/
/*************************************************************/
/*************************************************************/


/*************************************************************/
/* mode13: cette fonction permet de passer en mode graphique */
/*         13h par appel au bios.                            */
/*         elle permet aussi d'effacer l'cran en mode 13h   */
/*                                                           */
/* paramtre pass: aucun                                    */
/* valeur retourne: aucune                                  */
/*************************************************************/
void mode13(void)
{
  __dpmi_regs reg;
  _farsetsel(_dos_ds);  /* Permet l'accs  la mmoire basse */
  reg.x.ax=0x13;        /* Appel au bios, fonction 0 */
  __dpmi_int(0x10,&reg);
}

/*************************************************************/
/* mode3:  cette fonction permet de passer en mode texte 80  */
/*         colonnes par appel au bios.                       */
/*         elle permet aussi d'effacer l'cran texte         */
/*                                                           */
/* paramtre pass: aucun                                    */
/* valeur retourne: aucune                                  */
/*************************************************************/
void mode3(void)
{
  __dpmi_regs reg;
  reg.x.ax=3;           /* Appel au bios, fonction 0 */
  __dpmi_int(0x10,&reg);
}

/*************************************************************/
/* putpixel: cette fonction permet de dessiner un pixel en   */
/*         mode 13h                                          */
/*                                                           */
/* paramtres passs:                                        */
/*         x et y: coordonns du point                       */
/*         couleur: numro d'entre dans la palette          */
/* valeur retourne: aucune                                  */
/*************************************************************/
void putpixel(int x, int y, unsigned char couleur)
{
  /* Envoi de l'octet couleur en A0000+320y+x */
  _farnspokeb(0xA0000+x+(((y<<2)+y)<<6), couleur);
}

/*************************************************************/
/* getpixel: cette fonction permet de lire un pixel en mode  */
/*         13h                                               */
/*                                                           */
/* paramtres passs:                                        */
/*         x et y: coordonns du point                       */
/* valeur retourne:                                         */
/*         numro d'entre dans la palette                   */
/*************************************************************/
unsigned char getpixel(int x, int y)
{
  /* Lecture de l'octet en A0000+320y+x */
  return(_farnspeekb(0xA0000+x+(((y<<2)+y)<<6)));
}

/*************************************************************/
/* setcolor: cette fonction permet de programmer une valeur  */
/*         de la palette                                     */
/*                                                           */
/* paramtres passs:                                        */
/*         couleur: nunro d'entre dans la palette          */
/*         rouge, vert, bleu: composante (0  63)            */
/* valeur retourne: aucune                                  */
/*************************************************************/
void setcolor(unsigned char couleur, unsigned char rouge, unsigned char vert,
              unsigned char bleu)
{
  outportb(0x3C8,couleur);
  outportb(0x3C9,rouge);
  outportb(0x3C9,vert);
  outportb(0x3C9,bleu);
}

/*************************************************************/
/* setcolorsuite: cette fonction permet de programmer la     */
/*         valeur suivante de la palette (aprs avoir fait   */
/*         setcolor ou setcolorsuite).                       */
/*                                                           */
/* paramtres passs:                                        */
/*         rouge, vert, bleu: composante (0  63)            */
/* valeur retourne: aucune                                  */
/*************************************************************/
void setcolorsuite(unsigned char rouge, unsigned char vert,
                   unsigned char bleu)
{
  outportb(0x3C9,rouge);
  outportb(0x3C9,vert);
  outportb(0x3C9,bleu);
}

/*************************************************************/
/* vsync: cette fonction attend que le spot soit en bas.     */
/*        C'est utile dans certains graphiques rapides.      */
/*                                                           */
/* paramtre pass: aucun                                    */
/* valeur retourne: aucune                                  */
/*************************************************************/
void vsync(void)
{
  do {} while (inportb(0x3DA)&8); /* attend que le spot arrive en bas */
  do {} while (!(inportb(0x3DA)&8)); /* attend que le spot commence  monter */
}
